
/******************************************************************************/
/***  File:  defaults.c                                                     ***/
/***  Function:  This file reads the default parameters from .xtrekrc and   ***/
/***             sets appropriate flags.                                    ***/
/***                                                                        ***/
/***  Author:  Kevin P. Smith 6/11/89                                       ***/
/******************************************************************************/

#include "config.h"
#include "copyright2.h"
#include <stdio.h>
#include "Wlib.h"
#include "defs.h"
#include "struct.h"
#include "data.h"
#include <sys/file.h>

#include INC_STRINGS
#include INC_UNISTD
#include INC_CTYPE

struct stringlist
{
  char *string;
  char *value;
  struct stringlist *next;
};

struct stringlist *defaults = NULL;

char *getenv ();
int playerlistnum ();
extern unsigned char getctrlkey ();

/*************************************************************************/
/***  prob_desc[] will allow us to have simpler code to handle parsing ***/
/***  for these key words.                                             ***/
/*************************************************************************/
static char *prob_desc[] =
{"shld", "dam", "wtmp", "etmp", "arms", "fuel"};
static char *prob_severity[] =
{"low", "mid", "high"};

initDefaults (deffile)
     char *deffile;
{
  FILE *fp;
  char file[256];
  char *home;
  char *v;
  struct stringlist *new;
  struct dmacro_list *dm;
  struct dmacro_list *dm_def;
  int notdone;
  unsigned char c, *str;

  /* sizeof doesn't work if it isn't in the same source file, shoot me */
  MCOPY (dist_defaults, dist_prefered, sizedist);

  if (!deffile)
    {
      deffile = file;
      home = getenv ("HOME");
      if (!findDefaults (home, /*initialized */ deffile))
	{
	  if (home)
	    {
	      sprintf (file, "%s/.xtrekrc", home);
	    }
	  else
	    {
	      STRNCPY (file, ".xtrekrc", 9);
	    }
	  deffile = file;
	}
    }
  fp = fopen (deffile, "r");
  if (!fp)
    return;
  printf ("Reading defaults file %s\n", deffile);

#ifdef NBT
  macrocnt = 0;			/* reset macros */
#endif

  STRNCPY (defaultsFile, deffile, sizeof (defaultsFile));
  while (fgets (file, 250, fp))
    {
      if (*file == '#')
	continue;
      if (*file != 0)
	file[strlen (file) - 1] = 0;
      v = file;
      while (*v != ':' && *v != 0)
	{
	  v++;
	}
      if (*v == 0)
	continue;
      *v = 0;
      v++;
      while (*v == ' ' || *v == '\t')
	{
	  v++;
	}

#ifdef NBT
      /* not very robust but if it breaks nothing will die horribly I think - jmn */
      if (strncmpi (file, "macro.", 6) == 0)
	{
	  if (macrocnt == MAX_MACRO)
	    {
	      fprintf (stderr, "Maximum number of macros is %d\n", MAX_MACRO);
	    }
	  else
	    {
	      str = file + 6;
	      c = getctrlkey (&str);
	      if (c == '?')
		fprintf (stderr, "Cannot use '?' for a macro\n");
	      else
		{
		  macro[macrocnt].type = NBTM;
		  macro[macrocnt].key = c;
		  macro[macrocnt].who = str[1];
		  macro[macrocnt].string = strdup (v);
		  macrocnt++;
		}
	    }
	}
      else
#endif

      if (strncmpi (file, "mac.", 4) == 0)
	{
	  if (macrocnt == MAX_MACRO)
	    {
	      fprintf (stderr, "Maximum number of macros is %d\n", MAX_MACRO);
	    }
	  else
	    {
	      str = file + 4;
	      c = getctrlkey (&str);
	      if (c == '?')
		fprintf (stderr, "Cannot use '?' for a macro\n");
	      else
		{
		  macro[macrocnt].key = c;

		  if (str[0] == '.')
		    {
		      if (str[1] == '%')
			{
			  switch (str[2])
			    {
			    case 'u':
			    case 'U':
			    case 'p':
			      macro[macrocnt].who = MACRO_PLAYER;
			      break;
			    case 't':
			    case 'z':
			    case 'Z':
			      macro[macrocnt].who = MACRO_TEAM;
			      break;
			    case 'g':
			      macro[macrocnt].who = MACRO_FRIEND;
			      break;
			    case 'h':
			      macro[macrocnt].who = MACRO_ENEMY;
			      break;
			    default:
			      macro[macrocnt].who = MACRO_ME;
			      break;
			    }
			  macro[macrocnt].type = NEWMMOUSE;
			}
		      else
			{
			  macro[macrocnt].who = str[1];
			  macro[macrocnt].type = NEWMSPEC;
			}
		    }
		  else
		    {
		      macro[macrocnt].who = '\0';
		      macro[macrocnt].type = NEWM;
		    }
		  macro[macrocnt].string = strdup (v);
		  macrocnt++;
		}
	    }
	}

      else if (strncmpi (file, "dist.", 5) == 0)
	{
	  str = file + 5;
	  c = getctrlkey (&str);
	  if (*str != '.')
	    {
	      str = file + 4;
	      c = '\0';
	    }
	  str++;

	  notdone = 1;
	  for (dm = &dist_prefered[take], dm_def = &dist_defaults[take];
	       dm->name && notdone; dm++, dm_def++)

	    {
	      if (strcmpi (str, dm->name) == 0)
		{
		  dm->macro = strdup (v);
#ifdef DIST_KEY_NAME
		  if (c)
		    {
		      dm->c = c;
		      dm_def->c = c;
		    }
#endif /* DIST_KEY_NAME */
		  notdone = 0;
		}
	    }
	}

      else if (strncmpi (file, "singleMacro", 11) == 0)
	{
	  int i;
	  str = v;
	  for (i = 0; *str; i++)
	    singleMacro[i] = getctrlkey (&str);
	  singleMacro[i] = '\0';
	}

      if (*v != 0)
	{
	  new = (struct stringlist *) malloc (sizeof (struct stringlist));
	  new->next = defaults;
	  new->string = strdup (file);
	  new->value = strdup (v);
	  defaults = new;
	}
    }
  fclose (fp);
}

char *
getdefault (str)
     char *str;
{
  struct stringlist *sl;

  sl = defaults;
  while (sl != NULL)
    {
      if (strcmpi (sl->string, str) == 0)
	{
	  return (sl->value);
	}
      sl = sl->next;
    }
  return (NULL);
}


/* strcmpi tweaked 9/17/92 E-Mehlhaff to not tweak the strings its'
   ** called with...
   ** And tweaked again by NBT. Some systems have a demented strdup that
   ** doesn't put an end of string at the end and this causes no end of trouble...
 */
strcmpi (str1, str2)
     char *str1, *str2;
{
  char chr1, chr2;
  register int duh, stop;

  stop = strlen (str1);
  if (stop > strlen (str2))
    return 1;
  else if (stop < strlen (str2))
    return (-1);

  for (duh = 0; duh < stop; duh++)
    {
      chr1 = isupper (str1[duh]) ? str1[duh] : toupper (str1[duh]);
      chr2 = isupper (str2[duh]) ? str2[duh] : toupper (str2[duh]);
      if (chr1 == 0 || chr2 == 0)
	{
	  return (0);
	}
      if (chr1 != chr2)
	{
	  return (chr2 - chr1);
	}
    }
  return (0);
}


/* grr... are you telling me this sort of function isn't in
   ** the std libraries somewhere?! sons of satan... - jn
 */
strncmpi (str1, str2, max)
     char *str1, *str2;
     int max;
{
  char chr1, chr2;
  register int duh, stop;

  stop = strlen (str1);

  if (stop < max)
    return -1;

  if (stop > max)
    stop = max;

  if (stop > strlen (str2))
    return 1;

  for (duh = 0; duh < stop; duh++)
    {
      chr1 = isupper (str1[duh]) ? str1[duh] : toupper (str1[duh]);
      chr2 = isupper (str2[duh]) ? str2[duh] : toupper (str2[duh]);
      if (chr1 == 0 || chr2 == 0)
	{
	  return (0);
	}
      if (chr1 != chr2)
	{
	  return (chr2 - chr1);
	}
    }
  return (0);
}


booleanDefault (def, preferred)
     char *def;
     int preferred;
{
  char *str;

  str = getdefault (def);
  if (str == NULL)
    return (preferred);
  if (strcmpi (str, "on") == 0)
    {
      return (1);
    }
  else
    {
      return (0);
    }
}

intDefault (def, preferred)

     char *def;
     int preferred;
{
  char *str;
  str = getdefault (def);
  if (!str)
    return preferred;
  return atoi (str);
}

/* no default file given on command line.
   See if serverName is defined.  If it exists we look for
   HOME/.xtrekrc-<serverName> and .xtrekrc-<serverName>
   Otherwise we try DEFAULT_SERVER. */

/* since this is Find Defaults, I moved all the defaults file checking to
   ** it, and put in support for a system defaults file.
   ** and it uses the access() system call to determine if a defaults
   **  file exists.
   ** note, access() returns 0 if user can read file, -1 on error or if
   ** they can't.
   ** -EM
   **
   ** Is anyone else bothered by the fact that this writes to deffile
   ** without really knowing how much of deffile is allocated?
   **
 */

findDefaults (home, deffile)

     char *home, *deffile;
{
  int accessible = -1;

  /* first look for server specific .xtrekrc in $HOME */
  if (serverName)
    {
      if (home)
	sprintf (deffile, "%s/.xtrekrc-%s", home, serverName);
      else
	sprintf (deffile, ".xtrekrc-%s", serverName);

#ifdef DEBUG
      printf ("Looking for defaults file in %s\n", deffile);
#endif

      accessible = access (deffile, R_OK);
      if (accessible == 0)
	return 1;
      /* else return 0; */
    }

  /* then try default server server-speciric defaults file in $HOME */
  if (home)
    sprintf (deffile, "%s/.xtrekrc-%s", home, DEFAULT_SERVER);
  else
    sprintf (deffile, ".xtrekrc-%s", DEFAULT_SERVER);

#ifdef DEBUG
  printf ("Looking for defaults file in %s\n", deffile);
#endif

  accessible = access (deffile, R_OK);
  if (accessible == 0)
    return 1;

  /* now try .xtrekrc in $HOME */
  if (home)
    sprintf (deffile, "%s/.xtrekrc", home);
  else
    sprintf (deffile, ".xtrekrc");

#ifdef DEBUG
  printf ("Looking for defaults file in %s\n", deffile);
#endif

  accessible = access (deffile, R_OK);
  if (accessible == 0)
    return 1;

  /* and of course, try for a .netrekrc  in $HOME, for the new generation */
  /* now try .xtrekrc in $HOME */
  if (home)
    sprintf (deffile, "%s/.netrekrc", home);
  else
    sprintf (deffile, ".netrekrc");

#ifdef DEBUG
  printf ("Looking for defaults file in %s\n", deffile);
#endif

  accessible = access (deffile, R_OK);
  if (accessible == 0)
    return 1;

#ifdef SYSTEM_DEFAULTFILE
  /* now try for a system default defaults file */
  sprintf (deffile, SYSTEM_DEFAULTFILE);

#ifdef DEBUG
  printf ("Looking for defaults file in %s\n", deffile);
#endif

  accessible = access (deffile, R_OK);
  if (accessible == 0)
    return 1;
#endif

  return 0;
}

resetdefaults ()
{
  char *pek;
  char defaulttmp[100];

  keepInfo = intDefault ("keepInfo", keepInfo);
  showPlanetOwner = booleanDefault ("showPlanetOwner", showPlanetOwner);
  newDashboard = booleanDefault ("newDashboard", newDashboard);
  ROMVLVS = booleanDefault ("ROMVLVS", ROMVLVS);
  babes = booleanDefault ("babes", babes);
  updatespeed = intDefault ("updatespersec", updatespeed);
  logmess = booleanDefault ("logging", logmess);

#ifdef VSHIELD_BITMAPS
  VShieldBitmaps = booleanDefault ("varyShields", VShieldBitmaps);
#else
  warnShields = booleanDefault ("warnShields", warnShields);
#endif

  sortPlayers = booleanDefault ("sortPlayers", sortPlayers);

  netstat = booleanDefault ("netstats", netstat);
  netstatfreq = intDefault ("netstatfreq", netstatfreq);
  if (netstatfreq <= 0)
    netstatfreq = 1;

#ifdef RSA
  if (RSA_Client >= 0)
    {
      RSA_Client = booleanDefault ("useRsa", RSA_Client);
      sprintf (defaulttmp, "useRSA.%s", serverName);
      RSA_Client = booleanDefault (defaulttmp, RSA_Client);

    }
  else
    {
      /* RSA mode was specified in the command line args */
      RSA_Client = (RSA_Client == -2) ? 1 : 0;
    }
#endif

  myPlanetBitmap = booleanDefault ("newPlanetBitmaps", myPlanetBitmap);
#ifdef RABBIT_EARS
  if (myPlanetBitmap)
    myPlanetBitmap = intDefault ("whichNewPlanetBitmaps", myPlanetBitmap);
#endif
  showlocal = intDefault ("showlocal", showlocal);
  showgalactic = intDefault ("showgalactic", showgalactic);
  showLock = intDefault ("showLock", showLock);
  showPhaser = intDefault ("showPhaser", showPhaser);
  showShields = booleanDefault ("showshields", showShields);
  showStats = booleanDefault ("showstats", showStats);
  keeppeace = booleanDefault ("keeppeace", keeppeace);
  continuetractor = booleanDefault ("continuetractor", continuetractor);
  showTractorPressor = booleanDefault ("showTractorPressor", showTractorPressor);
  extraBorder = booleanDefault ("extraAlertBorder", extraBorder);
  namemode = booleanDefault ("showplanetnames", 1);
  reportKills = booleanDefault ("reportKills", reportKills);

  udpDebug = intDefault ("udpDebug", udpDebug);
  udpClientSend = intDefault ("udpClientSend", udpClientSend);
  /* note: requires send */
  udpClientRecv = intDefault ("udpClientReceive", udpClientRecv);
  tryUdp = booleanDefault ("tryUdp", tryUdp);
  tryUdp1 = tryUdp;
  udpSequenceChk = booleanDefault ("udpSequenceCheck", udpSequenceChk);

#ifdef SHORT_PACKETS
  tryShort = booleanDefault ("tryShort", tryShort);
  tryShort1 = tryShort;
#endif

  UseNewDistress = booleanDefault ("newDistress", UseNewDistress);
  rejectMacro = booleanDefault ("rejectMacro", rejectMacro);
  enemyPhasers = intDefault ("enemyPhasers", enemyPhasers);
  pek = getdefault ("cloakChars");
  if (pek != (char *) NULL)
    STRNCPY (cloakChars, pek, 3);
  showIND = booleanDefault ("showIND", showIND);
  newPlist = booleanDefault ("newPlist", newPlist);
  playerlistnum ();

#ifdef IGNORE_SIGNALS_SEGV_BUS
  if (ignore_signals >= 0)
    {
      ignore_signals = booleanDefault ("ignoreSignals", ignore_signals);

      if (ignore_signals)
	printf ("Ignoring signals SIGSEGV and SIGBUS\n");
    }
  else
    {
      /* ignoresignals mode was specified in the command line args */
      ignore_signals = (ignore_signals == -1) ? 1 : 0;
    }
#endif

#ifdef NEW_DASHBOARD_2
  cup_half_full = booleanDefault ("newdashboard2", cup_half_full);
  if (cup_half_full)
    newDashboard = cup_half_full;
#endif

#ifdef MOUSE_AS_SHIFT
  mouse_as_shift = booleanDefault ("mouseAsShift", mouse_as_shift);
#endif

#ifdef MOTION_MOUSE
  if (motion_mouse_enablable)
    motion_mouse = booleanDefault ("continuousMouse", motion_mouse);

  user_motion_thresh = intDefault ("motionThresh", user_motion_thresh);
#endif

#ifdef SHIFTED_MOUSE
  extended_mouse = booleanDefault ("shiftedMouse", extended_mouse);
#endif

#ifdef TNG_FED_BITMAPS
  use_tng_fed_bitmaps = booleanDefault ("useTNGBitmaps", use_tng_fed_bitmaps);
#endif

#ifdef VARY_HULL
  vary_hull = booleanDefault ("warnHull", vary_hull);
#endif

#ifdef SOUND
  sound_init = booleanDefault ("sound", sound_init);
#endif

}
