
/*
 * sound.c - Sound stuff
 */
#include "config.h"
#ifdef SOUND

#include "copyright.h"

#include <stdio.h>
#include INC_STDLIB
#include <ctype.h>
#include INC_STRINGS
#include INC_LIMITS
#include <time.h>
#include INC_SYS_TIME
#include "Wlib.h"
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "audio.h"

static char fire_torp_sound[PATH_MAX], cloaked_sound[PATH_MAX], shield_down_sound[PATH_MAX],
  shield_up_sound[PATH_MAX], explosion_sound[PATH_MAX], phaser_sound[PATH_MAX],
  torp_hit_sound[PATH_MAX], warning_sound[PATH_MAX], engine_sound[PATH_MAX],
  enter_ship_sound[PATH_MAX], message_sound[PATH_MAX], self_destruct_sound[PATH_MAX];

static int current_sound = NO_SOUND;

static int sound_toggle = 0;

static int st_cloaked = 1;
static int st_engine = 0;
static int st_enter_ship = 0;
static int st_explosion = 1;
static int st_fire_torp = 1;
static int st_message = 1;
static int st_phaser = 1;
static int st_red_alert = 1;
static int st_shield_down = 1;
static int st_shield_up = 1;
static int st_torp_hit = 1;
static int st_self_destruct = 1;

extern void
Exit_Sound ()
{
  if (sound_init)
    ExitSound ();
  sound_init = 0;
  sound_toggle = 0;
}

extern void
Init_Sound ()
{
  char intro_sound[PATH_MAX];

  if (sound_init)
    {
      if ((sounddir = getdefault ("sounddir")) == NULL)
	{
	  if (getenv ("SOUNDDIR") != NULL)
	{
	  sounddir = strdup (getenv ("SOUNDDIR"));
	}
      else
	sounddir = "./sounds";
	}

      if (InitSound () == -1)
	{
	  sound_toggle = 0;
	  sound_init = 0;
	}
      else
	sound_toggle = 1;

      strcpy (explosion_sound, sounddir);
      strcat (explosion_sound, "/nt_explosion");

      strcpy (enter_ship_sound, sounddir);
      strcat (enter_ship_sound, "/nt_enter_ship");

      strcpy (engine_sound, sounddir);
      strcat (engine_sound, "/nt_engine");

      strcpy (cloaked_sound, sounddir);
      strcat (cloaked_sound, "/nt_cloaked");

      strcpy (fire_torp_sound, sounddir);
      strcat (fire_torp_sound, "/nt_fire_torp");

      strcpy (message_sound, sounddir);
      strcat (message_sound, "/nt_message");

      strcpy (phaser_sound, sounddir);
      strcat (phaser_sound, "/nt_phaser");

      strcpy (warning_sound, sounddir);
      strcat (warning_sound, "/nt_warning");

      strcpy (shield_up_sound, sounddir);
      strcat (shield_up_sound, "/nt_shield_up");

      strcpy (shield_down_sound, sounddir);
      strcat (shield_down_sound, "/nt_shield_down");

      strcpy (torp_hit_sound, sounddir);
      strcat (torp_hit_sound, "/nt_torp_hit");

      strcpy (self_destruct_sound, sounddir);
      strcat (self_destruct_sound, "/nt_self_destruct");

      if (sound_toggle)
	{
	  strcpy (intro_sound, sounddir);
	  strcat (intro_sound, "/nt_intro");
	  StartSound (intro_sound);
	}
    }
}

extern void
Play_Sound (type)
     int type;
{
  switch (type)
    {
    case FIRE_TORP_SOUND:
      if (st_fire_torp && sound_toggle)
	StartSound (fire_torp_sound);
      break;

    case CLOAK_SOUND:
      if (st_cloaked && sound_toggle)
	StartSound (cloaked_sound);
      break;

    case SHIELD_DOWN_SOUND:
      if (st_shield_down && sound_toggle)
	StartSound (shield_down_sound);
      break;

    case SHIELD_UP_SOUND:
      if (st_shield_up && sound_toggle)
	StartSound (shield_up_sound);
      break;

    case EXPLOSION_SOUND:
      if (st_explosion && sound_toggle)
	StartSound (explosion_sound);
      break;

    case PHASER_SOUND:
      if (st_phaser && sound_toggle)
	StartSound (phaser_sound);
      break;

    case TORP_HIT_SOUND:
      if (st_torp_hit && sound_toggle)
	StartSound (torp_hit_sound);
      break;

    case WARNING_SOUND:
      if (st_red_alert && sound_toggle)
	StartSound (warning_sound);
      break;

    case ENGINE_SOUND:
      break;

    case ENTER_SHIP_SOUND:
      if (st_enter_ship && sound_toggle)
	StartSound (enter_ship_sound);
      break;

    case MESSAGE_SOUND:
      if (st_message && sound_toggle)
	StartSound (message_sound);
      break;

    case SELF_DESTRUCT_SOUND:
      if (st_self_destruct && sound_toggle)
	StartSound (self_destruct_sound);
      break;
    }

  if (type != ENGINE_SOUND)
    current_sound = type;

  if (!(sound_toggle))
    current_sound = NO_SOUND;

}

extern void
Abort_Sound (type)
     int type;
{
  if ((current_sound != NO_SOUND) && (type == current_sound))
    StopSound ();
}


/* Sound options window stuff */

#define SOUND_TOGGLE  0
#define CLOAKED       1
#define ENGINE        2
#define ENTER_SHIP    3
#define EXPLOSION     4
#define FIRE_TORP     5
#define MESSAGE       6
#define PHASER        7
#define RED_ALERT     8
#define SHIELD_DOWN   9
#define SHIELD_UP     10
#define TORP_HIT      11
#define SELF_DESTRUCT 12
#define SOUND_INIT    13
#define SOUND_DONE    14

static void soundrefresh ();

extern void
soundwindow ()
{
  int i;

  for (i = 0; i < NUM_SOUNDS + 3; i++)
    soundrefresh (i);

  /* Map window */
  W_MapWindow (soundWin);
}

static void
soundrefresh (i)
     int i;
{
  char buf[BUFSIZ];

  switch (i)
    {
    case SOUND_TOGGLE:
      sprintf (buf, "Sound is turned %s", (sound_toggle == 1) ? "ON" : "OFF");
      break;
    case CLOAKED:
      sprintf (buf, "Cloaked sound is %s", (st_cloaked == 1) ? "ON" : "OFF");
      break;
    case ENGINE:
      sprintf (buf, "Engine sound is not supported");
      /*    sprintf (buf, "Engine sound is %s", (st_engine==1) ? "ON" : "OFF"); */
      break;
    case ENTER_SHIP:
      sprintf (buf, "Enter ship sound is %s", (st_enter_ship == 1) ? "ON" :
	       "OFF");
      break;
    case EXPLOSION:
      sprintf (buf, "Explosion sound is %s", (st_explosion == 1) ? "ON" :
	       "OFF");
      break;
    case FIRE_TORP:
      sprintf (buf, "Fire torp sound is %s", (st_fire_torp == 1) ? "ON" :
	       "OFF");
      break;
    case MESSAGE:
      sprintf (buf, "Message sound is %s", (st_message == 1) ? "ON" : "OFF");
      break;
    case PHASER:
      sprintf (buf, "Phaser sound is %s", (st_phaser == 1) ? "ON" : "OFF");
      break;
    case RED_ALERT:
      sprintf (buf, "Red alert sound is %s", (st_red_alert == 1) ? "ON" :
	       "OFF");
      break;
    case SHIELD_DOWN:
      sprintf (buf, "Shield down  sound is %s", (st_shield_down == 1) ? "ON" :
	       "OFF");
      break;
    case SHIELD_UP:
      sprintf (buf, "Shield up sound is %s", (st_shield_up == 1) ? "ON" :

	       "OFF");
      break;
    case TORP_HIT:
      sprintf (buf, "Torp hit sound is %s", (st_torp_hit == 1) ? "ON" : "OFF");
      break;
    case SELF_DESTRUCT:
      sprintf (buf, "Self destruct sound is %s", (st_self_destruct == 1) ?
	       "ON" : "OFF");
      break;
    case SOUND_INIT:
      if (sound_init)
	strcpy (buf, "Restart external sound player");
      else
	strcpy (buf, "Initialize external sound player");
      break;
    case SOUND_DONE:
      strcpy (buf, "Done");
      break;
    default:
      fprintf (stderr, "Uh oh, bogus refresh number\n");
    }

  W_WriteText (soundWin, 0, i, textColor, buf, strlen (buf), 0);
}

extern void
soundaction (data)
     W_Event *data;
{
  switch (data->y)
    {
    case SOUND_TOGGLE:
      if (sound_init)
	sound_toggle = (sound_toggle == 1) ? 0 : 1;
      soundrefresh (SOUND_TOGGLE);
      if (!sound_toggle)
	{
	  Abort_Sound (ENGINE_SOUND);
	}
      else
	{
	  Play_Sound (ENGINE_SOUND);
	}
      break;

    case CLOAKED:
      st_cloaked = (st_cloaked == 1) ? 0 : 1;
      soundrefresh (CLOAKED);
      Play_Sound (CLOAK_SOUND);
      break;

    case ENGINE:
      st_engine = (st_engine == 1) ? 0 : 0;
      soundrefresh (ENGINE);
      if (st_engine && sound_toggle)
	{
	  Play_Sound (ENGINE_SOUND);
	}
      else
	{
	  Abort_Sound (ENGINE_SOUND);
	}
      break;

    case ENTER_SHIP:
      st_enter_ship = (st_enter_ship == 1) ? 0 : 1;
      soundrefresh (ENTER_SHIP);
      Play_Sound (ENTER_SHIP_SOUND);
      break;

    case EXPLOSION:
      st_explosion = (st_explosion == 1) ? 0 : 1;
      soundrefresh (EXPLOSION);
      Play_Sound (EXPLOSION_SOUND);
      break;

    case FIRE_TORP:
      st_fire_torp = (st_fire_torp == 1) ? 0 : 1;
      soundrefresh (FIRE_TORP);
      Play_Sound (FIRE_TORP_SOUND);
      break;

    case MESSAGE:
      st_message = (st_message == 1) ? 0 : 1;
      soundrefresh (MESSAGE);
      Play_Sound (MESSAGE_SOUND);
      break;

    case PHASER:
      st_phaser = (st_phaser == 1) ? 0 : 1;
      soundrefresh (PHASER);
      Play_Sound (PHASER_SOUND);
      break;

    case RED_ALERT:
      st_red_alert = (st_red_alert == 1) ? 0 : 1;
      soundrefresh (RED_ALERT);
      Play_Sound (WARNING_SOUND);
      break;

    case SHIELD_DOWN:
      st_shield_down = (st_shield_down == 1) ? 0 : 1;
      soundrefresh (SHIELD_DOWN);
      Play_Sound (SHIELD_DOWN_SOUND);
      break;

    case SHIELD_UP:
      st_shield_up = (st_shield_up == 1) ? 0 : 1;
      soundrefresh (SHIELD_UP);
      Play_Sound (SHIELD_UP_SOUND);
      break;

    case TORP_HIT:
      st_torp_hit = (st_torp_hit == 1) ? 0 : 1;
      soundrefresh (TORP_HIT);
      Play_Sound (TORP_HIT_SOUND);
      break;

    case SELF_DESTRUCT:
      st_self_destruct = (st_self_destruct == 1) ? 0 : 1;
      soundrefresh (SELF_DESTRUCT);
      Play_Sound (SELF_DESTRUCT_SOUND);
      break;

    case SOUND_INIT:
      Exit_Sound ();
      sound_init = 1;
      Init_Sound ();
      soundrefresh (SOUND_INIT);
      soundrefresh (SOUND_TOGGLE);
      break;

    case SOUND_DONE:
      sounddone ();
      break;
    }
}

extern void
sounddone ()
{
  W_UnmapWindow (soundWin);
}

#endif /* SOUND */
